@@ -9,15 +9,18 @@ from django.template.defaultfilters import escapejs
 from django.template.loader import render_to_string
 from django.utils.safestring import mark_safe
 from django.utils.translation import ugettext as _
+import os
 
 
 
+####################################################################################
+
 class AutoCompleteSelectWidget(forms.widgets.TextInput):
 
-    """  widget to select a model """
-    
+    """  widget to select a model and return it as text """
+
     add_link = None
-    
+
     def __init__(self,
                  channel,
                  help_text='',
@@ -39,23 +42,25 @@ class AutoCompleteSelectWidget(forms.widgets.TextInput):
                 obj = objs[0]
             except IndexError:
                 raise Exception("%s cannot find object:%s" % (lookup, value))
-            current_result = mark_safe(lookup.format_item( obj ) )
+            display = lookup.format_item_display(obj)
+            current_repr = mark_safe( """new Array("%s",%s)""" % (escapejs(display),obj.pk) )
         else:
-            current_result = ''
+            current_repr = 'null'
 
         context = {
                 'name': name,
                 'html_id' : self.html_id,
+                'min_length': getattr(lookup, 'min_length', 1),
                 'lookup_url': reverse('ajax_lookup',kwargs={'channel':self.channel}),
                 'current_id': value,
-                'current_result': current_result,
+                'current_repr': current_repr,
                 'help_text': self.help_text,
                 'extra_attrs': mark_safe(flatatt(final_attrs)),
                 'func_slug': self.html_id.replace("-",""),
                 'add_link' : self.add_link,
-                'admin_media_prefix' : settings.ADMIN_MEDIA_PREFIX
                 }
-
+        context.update(bootstrap())
+        
         return mark_safe(render_to_string(('autocompleteselect_%s.html' % self.channel, 'autocompleteselect.html'),context))
 
     def value_from_datadict(self, data, files, name):
@@ -66,6 +71,9 @@ class AutoCompleteSelectWidget(forms.widgets.TextInput):
         else:
             return None
 
+    def id_for_label(self, id_):
+        return '%s_text' % id_
+
 
 
 class AutoCompleteSelectField(forms.fields.CharField):
@@ -77,8 +85,9 @@ class AutoCompleteSelectField(forms.fields.CharField):
     def __init__(self, channel, *args, **kwargs):
         self.channel = channel
         widget = kwargs.get("widget", False)
+        
         if not widget or not isinstance(widget, AutoCompleteSelectWidget):
-            kwargs["widget"] = AutoCompleteSelectWidget(channel=channel,help_text=kwargs.get('help_text',_('Enter text to search.')))
+            kwargs["widget"] = AutoCompleteSelectWidget(channel=channel,help_text=kwargs.get('help_text') or _('Enter text to search.'))
         super(AutoCompleteSelectField, self).__init__(max_length=255,*args, **kwargs)
 
     def clean(self, value):
@@ -100,21 +109,24 @@ class AutoCompleteSelectField(forms.fields.CharField):
         _check_can_add(self,user,model)
 
 
+####################################################################################
+
 
 class AutoCompleteSelectMultipleWidget(forms.widgets.SelectMultiple):
 
     """ widget to select multiple models """
-    
+
     add_link = None
-    
+
     def __init__(self,
                  channel,
                  help_text='',
-                 show_help_text=False,#admin will also show help. set True if used outside of admin
+                 show_help_text=None,
                  *args, **kwargs):
         super(AutoCompleteSelectMultipleWidget, self).__init__(*args, **kwargs)
         self.channel = channel
-        self.help_text = help_text
+        
+        self.help_text = help_text or _('Enter text to search.')
         self.show_help_text = show_help_text
 
     def render(self, name, value, attrs=None):
@@ -127,7 +139,6 @@ class AutoCompleteSelectMultipleWidget(forms.widgets.SelectMultiple):
 
         lookup = get_lookup(self.channel)
 
-        current_name = "" # the text field starts empty
         # eg. value = [3002L, 1194L]
         if value:
             current_ids = "|" + "|".join( str(pk) for pk in value ) + "|" # |pk|pk| of current
@@ -139,35 +150,38 @@ class AutoCompleteSelectMultipleWidget(forms.widgets.SelectMultiple):
         # text repr of currently selected items
         current_repr_json = []
         for obj in objects:
-            repr = lookup.format_item(obj)
-            current_repr_json.append( """new Array("%s",%s)""" % (escapejs(repr),obj.pk) )
-
+            display = lookup.format_item_display(obj)
+            current_repr_json.append( """new Array("%s",%s)""" % (escapejs(display),obj.pk) )
         current_reprs = mark_safe("new Array(%s)" % ",".join(current_repr_json))
+        
         if self.show_help_text:
             help_text = self.help_text
         else:
             help_text = ''
-
+        
         context = {
             'name':name,
             'html_id':self.html_id,
+            'min_length': getattr(lookup, 'min_length', 1),
             'lookup_url':reverse('ajax_lookup',kwargs={'channel':self.channel}),
             'current':value,
-            'current_name':current_name,
             'current_ids':current_ids,
-            'current_reprs':current_reprs,
+            'current_reprs': current_reprs,
             'help_text':help_text,
             'extra_attrs': mark_safe(flatatt(final_attrs)),
             'func_slug': self.html_id.replace("-",""),
             'add_link' : self.add_link,
-            'admin_media_prefix' : settings.ADMIN_MEDIA_PREFIX
         }
+        context.update(bootstrap())
+
         return mark_safe(render_to_string(('autocompleteselectmultiple_%s.html' % self.channel, 'autocompleteselectmultiple.html'),context))
 
     def value_from_datadict(self, data, files, name):
         # eg. u'members': [u'|229|4688|190|']
         return [long(val) for val in data.get(name,'').split('|') if val]
 
+    def id_for_label(self, id_):
+        return '%s_text' % id_
 
 
 
@@ -179,11 +193,34 @@ class AutoCompleteSelectMultipleField(forms.fields.CharField):
 
     def __init__(self, channel, *args, **kwargs):
         self.channel = channel
-        help_text = kwargs.get('help_text',_('Enter text to search.'))
+
+        as_default_help = u'Enter text to search.'
+        help_text = kwargs.get('help_text')
+        if not (help_text is None):
+            try:
+                en_help = help_text.translate('en')
+            except AttributeError:
+                pass
+            else:
+                # monkey patch the django default help text to the ajax selects default help text
+                django_default_help = u'Hold down "Control", or "Command" on a Mac, to select more than one.'
+                if django_default_help in en_help:
+                    en_help = en_help.replace(django_default_help,'').strip()
+                    # probably will not show up in translations
+                    if en_help:
+                        help_text = _(en_help)
+                    else:
+                        help_text = _(as_default_help)
+        else:
+            help_text = _(as_default_help)
+
         # admin will also show help text, so by default do not show it in widget
         # if using in a normal form then set to True so the widget shows help
-        show_help_text = kwargs.get('show_help_text',False)
+        show_help_text = kwargs.pop('show_help_text',False)
+        
         kwargs['widget'] = AutoCompleteSelectMultipleWidget(channel=channel,help_text=help_text,show_help_text=show_help_text)
+        kwargs['help_text'] = help_text
+        
         super(AutoCompleteSelectMultipleField, self).__init__(*args, **kwargs)
 
     def clean(self, value):
@@ -195,6 +232,9 @@ class AutoCompleteSelectMultipleField(forms.fields.CharField):
         _check_can_add(self,user,model)
 
 
+####################################################################################
+
+
 class AutoCompleteWidget(forms.TextInput):
     """
     Widget to select a search result and enter the result as raw text in the text input field.
@@ -213,25 +253,31 @@ class AutoCompleteWidget(forms.TextInput):
     def render(self, name, value, attrs=None):
 
         value = value or ''
+        
         final_attrs = self.build_attrs(attrs)
         self.html_id = final_attrs.pop('id', name)
 
+        lookup = get_lookup(self.channel)
+
         context = {
-            'current_name': value,
+            'current_repr': mark_safe("'%s'" % escapejs(value)),
             'current_id': value,
             'help_text': self.help_text,
             'html_id': self.html_id,
+            'min_length': getattr(lookup, 'min_length', 1),
             'lookup_url': reverse('ajax_lookup', args=[self.channel]),
             'name': name,
             'extra_attrs':mark_safe(flatatt(final_attrs)),
-            'func_slug': self.html_id.replace("-","")
+            'func_slug': self.html_id.replace("-",""),
         }
+        context.update(bootstrap())
 
         templates = ('autocomplete_%s.html' % self.channel,
                      'autocomplete.html')
         return mark_safe(render_to_string(templates, context))
 
 
+
 class AutoCompleteField(forms.CharField):
     """
     Field uses an AutoCompleteWidget to lookup possible completions using a channel and stores raw text (not a foreign key)
@@ -249,21 +295,24 @@ class AutoCompleteField(forms.CharField):
         super(AutoCompleteField, self).__init__(*args, **defaults)
 
 
-
-
+####################################################################################
 
 def _check_can_add(self,user,model):
-    """ check if the user can add the model, deferring first to the channel if it implements can_add() \
-        else using django's default perm check. \
-        if it can add, then enable the widget to show the + link """
+    """ check if the user can add the model, deferring first to 
+        the channel if it implements can_add()
+        else using django's default perm check.
+        if it can add, then enable the widget to show the + link 
+    """
     lookup = get_lookup(self.channel)
-    try:
+    if hasattr(lookup,'can_add'):
         can_add = lookup.can_add(user,model)
-    except AttributeError:
+    else:
         ctype = ContentType.objects.get_for_model(model)
         can_add = user.has_perm("%s.add_%s" % (ctype.app_label,ctype.model))
     if can_add:
-        self.widget.add_link = reverse('add_popup',kwargs={'app_label':model._meta.app_label,'model':model._meta.object_name.lower()})
+        self.widget.add_link = reverse('add_popup',
+            kwargs={'app_label':model._meta.app_label,'model':model._meta.object_name.lower()})
+
 
 def autoselect_fields_check_can_add(form,model,user):
     """ check the form's fields for any autoselect fields and enable their widgets with + sign add links if permissions allow"""
@@ -272,3 +321,23 @@ def autoselect_fields_check_can_add(form,model,user):
             db_field = model._meta.get_field_by_name(name)[0]
             form_field.check_can_add(user,db_field.rel.to)
 
+
+def bootstrap():
+    b = {}
+    b['bootstrap'] = getattr(settings,'AJAX_SELECT_BOOTSTRAP',False)
+    inlines = getattr(settings,'AJAX_SELECT_INLINES',None)
+
+    b['inline'] = ''
+    if inlines == 'inline':
+        directory = os.path.dirname( os.path.realpath(__file__) )
+        f = open(os.path.join(directory,"static","css","ajax_select.css"))
+        css = f.read()
+        f = open(os.path.join(directory,"static","js","ajax_select.js"))
+        js = f.read()
+        b['inline'] = mark_safe(u"""<style type="text/css">%s</style><script type="text/javascript">//<![CDATA[%s//]]></script>""" % (css,js))
+    elif inlines == 'staticfiles':
+        b['inline'] = mark_safe("""<style type="text/css">@import url("%sajax_select/css/ajax_select.css");</style><script type="text/javascript" src="%sajax_select/js/ajax_select.js"></script>""" % (settings.STATIC_URL,settings.STATIC_URL))
+
+    return b
+
+
